import Mark from 'mark.js/dist/mark.min';

const elem = document.documentElement;
const urlParams = new URLSearchParams(window.location.search);
const tgId = urlParams.get('file');
const fullscreenButton = document.getElementById('fullscreen-toggle');
const edition = document.getElementById('edition');
const fassung = urlParams.get('edition');

/**
 * Open fullscreen
 *
 * @function
 */
const openFullscreen = function openFullscreen() {
  event.preventDefault();
  if (elem.requestFullscreen) {
    elem.requestFullscreen();
  } else if (elem.mozRequestFullScreen) {
    /* Firefox */
    elem.mozRequestFullScreen();
  } else if (elem.webkitRequestFullscreen) {
    /* Chrome, Safari and Opera */
    elem.webkitRequestFullscreen();
  } else if (elem.msRequestFullscreen) {
    /* IE/Edge */
    elem.msRequestFullscreen();
  }
};

/**
 *  Close fullscreen
 *
 * @function
 */
const closeFullscreen = function closeFullscreen() {
  if (document.exitFullscreen) {
    document.exitFullscreen();
  } else if (document.mozCancelFullScreen) {
    /* Firefox */
    document.mozCancelFullScreen();
  } else if (document.webkitExitFullscreen) {
    /* Chrome, Safari and Opera */
    document.webkitExitFullscreen();
  } else if (document.msExitFullscreen) {
    /* IE/Edge */
    document.msExitFullscreen();
  }
};

/**
 * This function triggers the appearence of the fullscreen button and the display
 * of header and footer depending on whether fullscreen is opened or closed.
 *
 * @function
 */
const triggerFullscreenButton = function triggerFullscreenButton() {
  const fullscreenIcon = document.getElementById('fullscreen-toggle').querySelector('[class*="icon-"]');
  const fullscreenTooltips = document.querySelectorAll('#tooltip-fullscreen span');
  fullscreenIcon.classList.toggle('icon-fullscreen');
  fullscreenIcon.classList.toggle('icon-collapse');
  fullscreenTooltips.forEach((e) => e.classList.toggle('hidden'));
  const footer = document.getElementById('footer');
  footer.style.display = footer.style.display === 'none' ? '' : 'none';
  const navbar = document.getElementById('header');
  navbar.style.display = navbar.style.display === 'none' ? '' : 'none';
};

/**
 * This function determines whether fullscreen is opened by checking the
 * appearance of the fullscreen button.
 *
 * @function
 */
const fullscreen = function fullscreen() {
  const fullscreenIcon = document.getElementById('fullscreen-toggle').querySelector('[class*="icon-"]');
  this.blur();
  if (fullscreenIcon.classList.contains('icon-fullscreen')) {
    openFullscreen();
  } else {
    closeFullscreen();
  }
};

/** This function is used for the event handling of clicking the fullscreen
 * button.
 *
 * @function
 */
const exitHandler = function exitHandler() {
  if (document.webkitIsFullScreen === false) {
    triggerFullscreenButton();
  } else if (document.mozFullScreen === false) {
    triggerFullscreenButton();
  } else if (document.msFullscreenElement === false) {
    triggerFullscreenButton();
  } else if (document.webkitIsFullScreen === true) {
    triggerFullscreenButton();
  } else if (document.mozFullScreen === true) {
    triggerFullscreenButton();
  } else if (document.msFullscreenElement === true) {
    triggerFullscreenButton();
  }
};

if (document.addEventListener) {
  document.addEventListener('webkitfullscreenchange', exitHandler, false);
  document.addEventListener('mozfullscreenchange', exitHandler, false);
  document.addEventListener('fullscreenchange', exitHandler, false);
  document.addEventListener('MSFullscreenChange', exitHandler, false);
}

/**
 * This function prepares the content of a modal displaying bibliographical data.
*
* @param {string} id An id of an object in the KMG Zotero-group
* @param {string} mode A string indicating editorial or authorial bibliographical data
* @function
*/
function setRefModalContent(id, mode) {
  const modal = document.getElementById('refModal');
  const loading = modal.getElementsByClassName('loading')[0];
  const error = modal.getElementsByClassName('error')[0];
  const content = modal.getElementsByClassName('content')[0];
  const citedRange = modal.getElementsByClassName('citedRange')[0];
  loading.style.display = 'block';
  content.style.display = 'none';
  citedRange.style.display = 'none';
  if (mode === 'KMG') {
    modal.getElementsByClassName('modal-title--KMG')[0].style.display = 'block';
    modal.getElementsByClassName('modal-title--base')[0].style.display = 'none';
  } else {
    modal.getElementsByClassName('modal-title--KMG')[0].style.display = 'none';
    modal.getElementsByClassName('modal-title--base')[0].style.display = 'block';
  }
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/zotero/items/${id}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText)[id];

      modal.getElementsByClassName('content')[0].innerHTML = object.bib;
      loading.style.display = 'none';
      content.style.display = 'block';
    } else if (this.status !== 200) {
      loading.style.display = 'none';
      error.style.display = 'block';
    }
  };
  xhttp.send();
};

function setRefModalContentCitedRange(id, mode, unit, from, to) {
  const modal = document.getElementById('refModal');
  const loading = modal.getElementsByClassName('loading')[0];
  const error = modal.getElementsByClassName('error')[0];
  const content = modal.getElementsByClassName('content')[0];
  const citedRange = modal.getElementsByClassName('citedRange')[0];
  loading.style.display = 'block';
  content.style.display = 'none';
  citedRange.style.display = 'none';
  if (mode === 'KMG') {
    modal.getElementsByClassName('modal-title--KMG')[0].style.display = 'block';
    modal.getElementsByClassName('modal-title--base')[0].style.display = 'none';
  } else {
    modal.getElementsByClassName('modal-title--KMG')[0].style.display = 'none';
    modal.getElementsByClassName('modal-title--base')[0].style.display = 'block';
  }

  const prefix = 'Hier: ';
  const singular = (from === to);
  let pages = '';
  if (!!from?.length & !!to?.length) {
    (singular) ? pages = from : pages = `${from}-${to}`;
  }
  else if (!!from?.length & unit=='page') {
    pages = `${from}ff`;
  }
  else {
    pages = 'invalid page format';
  }

  let unitType = '';
  switch (unit) {
    case null: break;
    case 'absatz': (singular) ? unitType='Absatz' : unitType='Absätze'; break;
    case 'chapter': unitType = 'Kapitel'; break;
    case 'column': (singular) ? unitType='Spalte' : unitType='Spalten'; break;
    case 'page': (singular) ? unitType='Seite' : unitType='Seiten'; break;
    case 'paragraph': unitType = '§'; break;
    case 'sheet': (singular) ? unitType='Blatt' : unitType='Blätter'; break;
    case 'vers': (singular) ? unitType='Vers' : unitType='Verse'; break;
    default: unitType = 'Error: no valid unit type detected'; break;
  }
  citedRange.innerHTML = `${prefix + unitType} ${pages}`;

  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/zotero/items/${id}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText)[id];
      modal.getElementsByClassName('content')[0].innerHTML = object.bib;
      loading.style.display = 'none';
      content.style.display = 'block';
      citedRange.style.display = 'block';
    } else if (this.status !== 200) {
      loading.style.display = 'none';
      error.style.display = 'block';
      citedRange.style.display = 'none';
    }
  };
  xhttp.send();
};

/**
 * This function inserts the metadata of the current edition into the info
 * modal.
*
* @function
*/
const getMetadata = function getMetadata() {
  const infoModalBody = document.getElementById('infoModal').querySelector('.modal-body');
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/api/meta/${tgId}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText);
      infoModalBody.querySelector('#editionMainTitle').innerHTML = object.title;
      infoModalBody.querySelector('#edition-dateCreated').innerHTML = object.created.string;
      infoModalBody.querySelector('#edition-dateModified').innerHTML = object.modified.string;
      infoModalBody.querySelector('#edition-dateTransformed').innerHTML = object.transformed.string;
      infoModalBody.querySelector('#edition-textGridURI').innerHTML = object.tguri;
    }
  };
  xhttp.send();
};

/**
 * This function toggles the visibility of the given element.
*
* @function
*/
function toggleElementVisibility(elementID) {
  const x = document.getElementById(elementID);
  if (x.style.display === 'none') {
    x.style.display = 'block';
  } else {
    x.style.display = 'none';
  }
}

/**
 * This function checks the visibility of the given element.
*
* @function
*/
function isElementVisible(elementID) {
  const x = document.getElementById(elementID);
  if (window.getComputedStyle(x).display !== 'none') {
    return true;
  }
  return false;
}

/**
 * This function inserts the bibliographix metadata of the current edition into the info
 * modal.
*
* @function
*/
function getWitnessData() {
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/api/data/${tgId}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText);
      let editionDesc = '';
      let zotero = '';
      switch (urlParams.get('edition')) {
        case 'digital':
          editionDesc = 'Textkritisch-interaktive Fassung';
          break;
        case 'tia':
          editionDesc = 'Textkritisch-interaktive Fassung';
          break;
        case 'commentary':
          editionDesc = 'Werkkommentar';
          break;
        default:
          editionDesc = object.witnesses[urlParams.get('edition')].description;
          //TODO: need to replace link when zotero api is ready.
          zotero = object.witnesses[urlParams.get('edition')].zotero;
          break;
      }
      document.getElementById('editionBiblInfoDesc').innerHTML = editionDesc;
      if ((zotero != '') && (zotero != 'null') && (zotero != null)) {
        toggleElementVisibility('editionBiblInfoOrigPublSection');
        document.getElementById('editionBiblInfoOrigPubl').href = zotero;
      }
      else if (isElementVisible('editionBiblInfoOrigPublSection')) {
        toggleElementVisibility('editionBiblInfoOrigPublSection');
      }
    }
  };
  xhttp.send();
};

/**
   * This function hides any items in the info modal's legend if there are no
   * corresponding instances in the current edition.
  *
  * @function
  */
const checkUsedSymbols = function checkUsedSymbols() {
  const infoModalBody = document.getElementById('infoModal').querySelector('.modal-body');
  const legend = infoModalBody.querySelector('.js-edition-info-section--legend');
  const iconClasses = ['.correction', '.js-editorial-info--bibl-KMG', '.js-editorial-info--bibl', '.reference', '.js-editorial-info--note', '.abbreviation'];
  iconClasses.forEach((iconClass) => (edition.querySelector(iconClass) ? '' : legend.querySelector(iconClass).style.display = 'none'));
};

/**
   * This function copies text referenced by the element's @data-clipboard-target to the
   * clipboard
  *
  * @function
  */
const copyToClipboard = function copyToClipboard() {
  const targetId = this.getAttribute('data-clipboard-target').replace('#', '');
  const targetText = document.getElementById(targetId).innerHTML;
  this.blur();
  navigator.clipboard.writeText(targetText);
};

/**
 * This function determines the  vertical position of editorial content,
 * either at the same height as its trigger or below any other open
 * editorial content.
 *
 * @param {string} id
 * @param {integer} height The offset of the element that triggered the event
 */
const positionEditorial = function positionEditorial(id, height) {
  const thisBox = document.querySelector(`[data-ref="${id}"]`);
  const openBoxes = document.querySelectorAll('.editorial-content-show');
  const thisBoxHeight = thisBox.offsetHeight;
  let thisBoxBottom = height + thisBox.offsetHeight;
  let thisBoxTop = height;
  const boxesArray = [...openBoxes].sort((a, b) => a.offsetTop - b.offsetTop);

  boxesArray.forEach((otherBox) => {
    if (otherBox !== thisBox) {
      const otherBoxTop = otherBox.offsetTop;
      const otherBoxBottom = otherBoxTop + otherBox.offsetHeight;
      if (thisBoxTop >= otherBoxBottom + 5 || thisBoxBottom <= otherBoxTop - 5) { return; }
      if (
        (thisBoxTop >= otherBoxTop - 5 && thisBoxTop <= otherBoxBottom + 5)
        || (thisBoxBottom >= otherBoxTop - 5 && thisBoxBottom <= otherBoxBottom + 5)
      ) {
        thisBoxTop = otherBoxBottom + 5;
        thisBoxBottom = thisBoxTop + thisBoxHeight;
      }
    }
  });

  thisBox.style = `--offset-top: ${thisBoxTop}px`;
};

const highlightQuote = function highlightCorrespQuote(node, highlight) {
  const highlighted = node.classList.contains('highlight-quote');
  if (highlight) {
    node.classList.add('highlight-quote');
  } else {
    node.classList.remove('highlight-quote');
  }
  if (!highlighted && node.dataset.corresp) {
    const corresp = node.dataset.corresp.split(' ');
    [...corresp].forEach((e) => {
      const element = document.querySelector(`[data-corresp*="${e}"]`);
      highlightQuote(element, highlight);
    });
  }
};

const toggleEditorial = function toggleEditorial(element) {
  let infoBox;
  let id;
  let references;
  let offsetTop;
  let targetOffset;
  // Click on dismiss button will just hide anything
  if (element.classList.contains('index-entry-dismiss')) {
    infoBox = element.closest('.editorial-content');
    [id] = infoBox.dataset.ref.split(' ');
    references = document.querySelectorAll(`[data-target="${id}"]`);
    infoBox.classList.toggle('hidden');
    infoBox.classList.toggle('editorial-content-show');
    references.forEach((e) => e.classList.toggle('highlight-info'));
    return;
  }
  [id] = element.dataset.target.split(' ');
  infoBox = document.querySelector(`[data-ref="${id}"]`);
  // if the editorial is triggered from within another editorial,
  // offsetTop is calculated from the parent editorial without substracting
  // the container's offsetTop, as the editorials are positioned absolutely
  if (element.closest('.editorial-content')) {
    offsetTop = element.closest('.editorial-content').offsetTop;
    targetOffset = offsetTop;
  } else {
    offsetTop = element.offsetTop;
    targetOffset = offsetTop;
  }
  references = document.querySelectorAll(`[data-target="${id}"]`);
  infoBox.classList.toggle('hidden');
  infoBox.classList.toggle('editorial-content-show');
  references.forEach((e) => e.classList.toggle('highlight-info'));
  if (!infoBox.classList.contains('hidden')) {
    positionEditorial(id, targetOffset);
  }
  if (element.parentElement.classList.contains('linked-quote')) {
    const activateHighlight = !element.parentElement.classList.contains('highlight-quote');
    highlightQuote(element.parentElement, activateHighlight);
  }
};

const openEditorial = function openEditorial(event) {
  event.stopPropagation();
  toggleEditorial(this);
};

const pinEditorial = function pinEditorial(event) {
  event.stopPropagation();
  document.querySelectorAll('.pinned').forEach((e) => {
    if (e !== this) e.classList.remove('pinned');
  });
  this.classList.toggle('pinned');
};

const loadReference = function loadReference(event) {
  event.stopPropagation();
  const [id] = this.dataset.target.split(' ');
  const infoBox = document.querySelector(`[data-ref="${id}"]`);
  if (infoBox.dataset.resolved === 'true') return;
  const errorMessage = infoBox.querySelector('.load-error');
  errorMessage.classList.add('hidden');
  errorMessage.setAttribute('aria-hidden', 'true');
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/zotero/items/${id}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText)[id];

      infoBox.querySelector('.editorial-content-body').innerHTML = object.bib;
      infoBox.dataset.resolved = 'true';
    } else if (this.status !== 200) {
      errorMessage.classList.remove('hidden');
      errorMessage.setAttribute('aria-hidden', 'false');
    }
  };
  xhttp.send();
};

const scrollToEdition = function scrollToRequestedEntityInEdition() {
  const url = new URL(window.location.href);
  const id = url.searchParams.get('entity');
  if (id) {
    const entity = document.querySelector(`.entity-name[data-target="${id}"]`);
    if (entity) {
      // entity.scrollIntoView();
      window.scrollTo({ top: entity.offsetTop - 64 });
      toggleEditorial(entity);
    }
  }
};

const downloadButton = document.querySelector('.js-edition-menu__download');

/**
 * This function sets the @download and @href of the download button to the full edition according to the current
 * edition.
*
* @function
*/
const setDownloadLink = function setDownloadLink() {
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/api/meta/${tgId}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState == 4 && this.status == 200) {
      const object = JSON.parse(this.responseText);
      downloadButton.download = `Mollenhauer:${object.title}_${tgId}.xml`;
    }
  };
  xhttp.send();
};

if (downloadButton) {
  setDownloadLink();
}

const downloadButtonCitation = document.querySelector('.js-edition-cite__download');

/**
 * This function sets the @download and @href of the download button to the citation
 * according to the current edition.
*
* @function
*/
const setDownloadCitationLink = function setDownloadCitationLink() {
  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/api/getCitation/${tgId}/${fassung}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = this.responseText;
      downloadButtonCitation.download = `Mollenhauer:${object.title}_${tgId}_${fassung}.bib`;
    }
  };
  xhttp.send();
  downloadButtonCitation.href = `/exist/restxq/api/getCitation/${tgId}/${fassung}`;
};

if (downloadButtonCitation) {
  setDownloadCitationLink();
}

const downloadButtonPlain = document.querySelector('.js-edition-menu__download__plain');

/**
 * This function sets the @download and @href of the download button to the full edition
 * according to the current edition.
*
* @function
*/
const setDownloadPlainLink = function setDownloadPlainLink() {
  downloadButtonPlain.href = `https://textgridlab.org/1.0/aggregator/text/textgrid:${tgId}`;
};

if (downloadButtonPlain) {
  setDownloadPlainLink();
}

const clipboardButtons = document.querySelectorAll('.clipboard-btn');
clipboardButtons.forEach((button) => button.addEventListener('click', copyToClipboard));

/**
 * This function assigns the necessary events to any icons in edition texts.
*
* @function
*/
const assignTextEvents = function assignTextEvents() {
  const corrIcons = document.querySelectorAll('.js-editorial-info--corr');
  const abbrIcons = document.querySelectorAll('.js-editorial-info--abbr');
  const personIcons = document.querySelectorAll('.js-editorial-info--person');
  const biblKMGIcons = document.querySelectorAll('.js-editorial-info--bibl-KMG');
  const biblIcons = document.querySelectorAll('.js-editorial-info--bibl');
  const noteIcons = document.querySelectorAll('.js-editorial-info--note');
  const orgIcons = document.querySelectorAll('.js-editorial-info--org');
  const artIcons = document.querySelectorAll('.js-editorial-info--art');
  const periodicalIcons = document.querySelectorAll('.js-editorial-info--periodica');
  const musicIcons = document.querySelectorAll('.js-editorial-info--music');
  const brandIcons = document.querySelectorAll('.js-editorial-info--brand');
  const editorialContentDismiss = document.querySelectorAll('.index-entry-dismiss');
  const pinButtons = document.querySelectorAll('.button-pin');
  pinButtons.forEach((e) => { e.addEventListener('click', pinEditorial); });

  editorialContentDismiss.forEach((e) => { e.addEventListener('click', openEditorial); });
  corrIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  abbrIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  personIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  biblKMGIcons.forEach((e) => { e.addEventListener('click', loadReference); });
  biblIcons.forEach((e) => { e.addEventListener('click', loadReference); });
  biblKMGIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  biblIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  noteIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  orgIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  artIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  periodicalIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  musicIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
  brandIcons.forEach((e) => { e.addEventListener('click', openEditorial); });
};

document.addEventListener('DOMContentLoaded', assignTextEvents);
scrollToEdition();

const highlightSearch = function highlightSearchResults() {
  if (urlParams.get('search')) {
    const markInstance = new Mark(document.querySelector('.edition-text'));
    let keyword = urlParams.get('search');
    if (keyword.startsWith('"') && keyword.endsWith('"')) {
      keyword = keyword.slice(1).slice(0, -1);
    }
    const options = {
      acrossElements: true,
      className: 'highlight-search',
      separateWordSearch: false,
    };
    markInstance.mark(keyword, options);
  }
};

document.addEventListener('DOMContentLoaded', highlightSearch);

/**
 * This function delivers todays date in the format MM/DD/YYYY.
*
* @function
*/
function getTodaysDate() {
  let today = new Date();
  const dd = String(today.getDate()).padStart(2, '0');
  const mm = String(today.getMonth() + 1).padStart(2, '0'); //January is 0!
  const yyyy = today.getFullYear();

  today = `${dd}.${mm}.${yyyy}`;
  return today;
}

/**
 * This function is a helper function of getCitationData() that helps to get a timespan of years.
*
* @param {object} witnesses JSON object that includes years
* @function
*/
function getTimeSpan(witnesses) {
  const array = Object.keys(witnesses);
  const allYears = [];
  array.forEach((witness) => allYears.push(witnesses[witness].year));
  const lowestYear = Math.min(...allYears);
  const highestYear = Math.max(...allYears);
  const output = `${lowestYear.toString()}-${highestYear.toString()}`;
  return output;
}

/**
 * This function gets all required information about the citation and sets it to the info modal.
*
* @function
*/
const getCitationData = function getCitationData() {
  const partCitePublisher = ' In <i>Klaus Mollenhauer Gesamtausgabe. Historisch-kritische Edition</i> (2024). Hrsg. von Cornelie Dietrich, Klaus-Peter Horn &amp; Hans-Rüdiger Müller. ';
  const partCiteURL = document.URL;
  const partCiteDate = getTodaysDate();
  const citationEnd = `${partCitePublisher + partCiteURL}. Zugegriffen: ${partCiteDate}.`;
  document.getElementById('infoCiteEnd').innerHTML = citationEnd;

  const xhttp = new XMLHttpRequest();
  xhttp.open('GET', `/exist/restxq/api/data/${tgId}`, true);
  xhttp.onreadystatechange = function request() {
    if (this.readyState === 4 && this.status === 200) {
      const object = JSON.parse(this.responseText);
      const author = `${object.authors.surname}, ${object.authors.forename}`;
      document.getElementById('infoCiteAuthors').innerHTML = author;

      let editionNr = ` (${object.editionnr}`;
      let editionShort = '';
      let editionDesc = '';
      let editionJahre = '';
      switch (urlParams.get('edition')) {
        case 'digital':
          editionShort = 'tia';
          editionDesc = 'textkritisch-interaktive Ansicht ';
          editionJahre = getTimeSpan(object.witnesses);
          break;
        case 'tia':
          editionShort = 'tia';
          editionDesc = 'textkritisch-interaktive Ansicht ';
          editionJahre = getTimeSpan(object.witnesses);
          break;
        case 'commentary':
          editionShort = 'sta';
          editionDesc = 'Studienausgabe ';
          editionJahre = getTimeSpan(object.witnesses);
          break;
        default:
          editionShort = urlParams.get('edition');
          editionDesc = object.witnesses[urlParams.get('edition')].description;
          editionJahre = '';
          break;
      }
      editionNr = `${editionNr}-${editionShort}). `;
      document.getElementById('infoCiteEditionNr').innerHTML = editionNr;
      document.getElementById('infoCiteTitle').innerHTML = `${object.title.main}. ${object.title.sub}`;
      document.getElementById('infoCiteDesc').innerHTML = ` (${editionDesc}${editionJahre}).`;
    }
  };
  xhttp.send();
};

/**
         * This function opens the info modal.
        *
        * @param event
        * @function
        */
const openInfoModal = function openInfoModal() {
  event.preventDefault();
  $('#infoModal').modal('toggle');
  getMetadata();
  checkUsedSymbols();
  getWitnessData();
  getCitationData();
};

/**
 * add event listeners
 */
const infoButton = document.querySelector('.js-edition-menu__info');

infoButton?.addEventListener('click', openInfoModal);
fullscreenButton?.addEventListener('click', fullscreen);
